# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from . import config_type, config_schema, config_instance, config_instance_deploy_response
from .. import _compat
from .base_device import BaseDevice as BaseDevice
from .config_type import ConfigType as ConfigType
from .config_schema import ConfigSchema as ConfigSchema
from .paginated_list import PaginatedList as PaginatedList
from .config_instance import ConfigInstance as ConfigInstance
from .config_schema_list import ConfigSchemaList as ConfigSchemaList
from .device_list_params import DeviceListParams as DeviceListParams
from .device_create_params import DeviceCreateParams as DeviceCreateParams
from .device_list_response import DeviceListResponse as DeviceListResponse
from .device_update_params import DeviceUpdateParams as DeviceUpdateParams
from .config_instance_list_params import ConfigInstanceListParams as ConfigInstanceListParams
from .config_instance_deploy_params import ConfigInstanceDeployParams as ConfigInstanceDeployParams
from .config_instance_list_response import ConfigInstanceListResponse as ConfigInstanceListResponse
from .config_instance_reject_params import ConfigInstanceRejectParams as ConfigInstanceRejectParams
from .config_instance_approve_params import ConfigInstanceApproveParams as ConfigInstanceApproveParams
from .config_instance_deploy_response import ConfigInstanceDeployResponse as ConfigInstanceDeployResponse
from .config_instance_retrieve_params import ConfigInstanceRetrieveParams as ConfigInstanceRetrieveParams
from .device_create_activation_token_response import (
    DeviceCreateActivationTokenResponse as DeviceCreateActivationTokenResponse,
)

# Rebuild cyclical models only after all modules are imported.
# This ensures that, when building the deferred (due to cyclical references) model schema,
# Pydantic can resolve the necessary references.
# See: https://github.com/pydantic/pydantic/issues/11250 for more context.
if _compat.PYDANTIC_V2:
    config_instance.ConfigInstance.model_rebuild(_parent_namespace_depth=0)
    config_schema.ConfigSchema.model_rebuild(_parent_namespace_depth=0)
    config_type.ConfigType.model_rebuild(_parent_namespace_depth=0)
    config_instance_deploy_response.ConfigInstanceDeployResponse.model_rebuild(_parent_namespace_depth=0)
else:
    config_instance.ConfigInstance.update_forward_refs()  # type: ignore
    config_schema.ConfigSchema.update_forward_refs()  # type: ignore
    config_type.ConfigType.update_forward_refs()  # type: ignore
    config_instance_deploy_response.ConfigInstanceDeployResponse.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .base_device import BaseDevice

__all__ = ["ConfigInstance"]


class ConfigInstance(BaseModel):
    id: str
    """ID of the config instance"""

    activity_status: Literal["created", "validating", "validated", "queued", "deployed", "removed"]
    """Last known activity state of the config instance

    - Created: config instance has been created but no deployment attempt has been
      made
    - Validating: config instance is being validated with user's custom validation
    - Validated: config instance has been validated with user's custom validation
    - Queued: config instance will be deployed as soon as the device is back online
    - Deployed: config instance is currently available on the device
    - Removed: the config instance is no longer available on the device
    """

    config_schema: Optional["ConfigSchema"] = None
    """Expand the config schema using 'expand[]=config_schema' in the query string"""

    config_schema_id: str
    """ID of the config schema which the config instance must adhere to"""

    config_type: Optional["ConfigType"] = None
    """Expand the config type using 'expand[]=config_type' in the query string"""

    config_type_id: str
    """ID of the config type which the config instance (and its schema) is a part of"""

    content: Optional[object] = None
    """The configuration values associated with the config instance.

    Expand the content using 'expand[]=content' in the query string
    """

    created_at: datetime
    """The timestamp of when the config instance was created"""

    device: Optional[BaseDevice] = None

    device_id: str
    """ID of the device which the config instance is deployed to"""

    error_status: Literal["none", "failed", "retrying"]
    """Last known error state of the config instance deployment

    - None: there are no errors with the config instance deployment
    - Retrying: an error has been encountered and the agent is attempting to try
      again to reach the target status
    - Failed: an error has been encountered but no more retries are left; the config
      instance is removed (if deployed)
    """

    object: Literal["config_instance"]

    relative_filepath: str
    """
    The file path to deploy the config instance relative to
    `/srv/miru/config_instances`. `v1/motion-control.json` would deploy to
    `/srv/miru/config_instances/v1/motion-control.json`
    """

    status: Literal["created", "validating", "validated", "queued", "deployed", "removed", "failed", "retrying"]
    """
    This status merges the 'activity_status' and 'error_status' fields, with error
    states taking precedence over activity states when errors are present. For
    example, if the activity status is 'deployed' but there's an error, the overall
    status will be 'failed' or 'retrying' depending on the error state.
    """

    target_status: Literal["created", "validated", "deployed", "removed"]
    """Desired state of the config instance

    - Created: config instance desires not to be deployed but can be deployed in the
      future
    - Validated: config instance desires user's custom validation before deployment
      can begin
    - Deployed: config instance desires to be available on the device
    - Removed: config instance desires to no longer be available on the device
    """

    updated_at: datetime
    """The timestamp of when the config instance was last updated"""


from .config_type import ConfigType
from .config_schema import ConfigSchema



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["ConfigInstanceRetrieveParams"]


class ConfigInstanceRetrieveParams(TypedDict, total=False):
    expand: List[Literal["content", "config_schema", "device", "config_type"]]
    """The fields to expand in the config instance"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["ConfigInstanceListParams"]


class ConfigInstanceListParams(TypedDict, total=False):
    id: str
    """The config instance ID to filter by"""

    activity_status: Literal["created", "validating", "validated", "queued", "deployed", "removed"]
    """The config instance activity status to filter by"""

    config_schema_id: str
    """The config schema ID to filter by"""

    config_type_id: str
    """The config type ID to filter by"""

    device_id: str
    """The device ID to filter by"""

    error_status: Literal["none", "failed", "retrying"]
    """The config instance error status to filter by"""

    expand: List[Literal["total_count", "content", "config_schema", "device", "config_type"]]
    """The fields to expand in the config instance list"""

    limit: int
    """The maximum number of items to return.

    A limit of 15 with an offset of 0 returns items 1-15.
    """

    offset: int
    """The offset of the items to return.

    An offset of 10 with a limit of 10 returns items 11-20.
    """

    order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]

    target_status: Literal["created", "validated", "deployed", "removed"]
    """The config instance target status to filter by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ..types import device_list_params, device_create_params, device_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.base_device import BaseDevice
from ..types.device_list_response import DeviceListResponse
from ..types.device_create_activation_token_response import DeviceCreateActivationTokenResponse

__all__ = ["DevicesResource", "AsyncDevicesResource"]


class DevicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/miruml/python-server-sdk#accessing-raw-response-data-eg-headers
        """
        return DevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/miruml/python-server-sdk#with_streaming_response
        """
        return DevicesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """
        Create a new device

        Args:
          name: The name of the device

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/devices",
            body=maybe_transform({"name": name}, device_create_params.DeviceCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    def retrieve(
        self,
        device_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """
        Get a device by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._get(
            f"/devices/{device_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    def update(
        self,
        device_id: str,
        *,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """Update a device by ID

        Args:
          name: The new name of the device.

        If null or excluded from the request, the device
              name is not updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._patch(
            f"/devices/{device_id}",
            body=maybe_transform({"name": name}, device_update_params.DeviceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    def list(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        expand: List[Literal["total_count"]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeviceListResponse:
        """
        List devices

        Args:
          id: The device ID to filter by

          expand: The fields to expand in the device list

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          name: The device name to filter by

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/devices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "name": name,
                        "offset": offset,
                        "order_by": order_by,
                    },
                    device_list_params.DeviceListParams,
                ),
            ),
            cast_to=DeviceListResponse,
        )

    def create_activation_token(
        self,
        device_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeviceCreateActivationTokenResponse:
        """
        Create a new device activation token

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._post(
            f"/devices/{device_id}/activation_token",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeviceCreateActivationTokenResponse,
        )


class AsyncDevicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/miruml/python-server-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/miruml/python-server-sdk#with_streaming_response
        """
        return AsyncDevicesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """
        Create a new device

        Args:
          name: The name of the device

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/devices",
            body=await async_maybe_transform({"name": name}, device_create_params.DeviceCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    async def retrieve(
        self,
        device_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """
        Get a device by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._get(
            f"/devices/{device_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    async def update(
        self,
        device_id: str,
        *,
        name: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BaseDevice:
        """Update a device by ID

        Args:
          name: The new name of the device.

        If null or excluded from the request, the device
              name is not updated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._patch(
            f"/devices/{device_id}",
            body=await async_maybe_transform({"name": name}, device_update_params.DeviceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BaseDevice,
        )

    async def list(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        expand: List[Literal["total_count"]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeviceListResponse:
        """
        List devices

        Args:
          id: The device ID to filter by

          expand: The fields to expand in the device list

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          name: The device name to filter by

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/devices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "expand": expand,
                        "limit": limit,
                        "name": name,
                        "offset": offset,
                        "order_by": order_by,
                    },
                    device_list_params.DeviceListParams,
                ),
            ),
            cast_to=DeviceListResponse,
        )

    async def create_activation_token(
        self,
        device_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeviceCreateActivationTokenResponse:
        """
        Create a new device activation token

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._post(
            f"/devices/{device_id}/activation_token",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeviceCreateActivationTokenResponse,
        )


class DevicesResourceWithRawResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

        self.create = to_raw_response_wrapper(
            devices.create,
        )
        self.retrieve = to_raw_response_wrapper(
            devices.retrieve,
        )
        self.update = to_raw_response_wrapper(
            devices.update,
        )
        self.list = to_raw_response_wrapper(
            devices.list,
        )
        self.create_activation_token = to_raw_response_wrapper(
            devices.create_activation_token,
        )


class AsyncDevicesResourceWithRawResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

        self.create = async_to_raw_response_wrapper(
            devices.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            devices.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            devices.update,
        )
        self.list = async_to_raw_response_wrapper(
            devices.list,
        )
        self.create_activation_token = async_to_raw_response_wrapper(
            devices.create_activation_token,
        )


class DevicesResourceWithStreamingResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

        self.create = to_streamed_response_wrapper(
            devices.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            devices.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            devices.update,
        )
        self.list = to_streamed_response_wrapper(
            devices.list,
        )
        self.create_activation_token = to_streamed_response_wrapper(
            devices.create_activation_token,
        )


class AsyncDevicesResourceWithStreamingResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

        self.create = async_to_streamed_response_wrapper(
            devices.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            devices.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            devices.update,
        )
        self.list = async_to_streamed_response_wrapper(
            devices.list,
        )
        self.create_activation_token = async_to_streamed_response_wrapper(
            devices.create_activation_token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..types import (
    config_instance_list_params,
    config_instance_deploy_params,
    config_instance_reject_params,
    config_instance_approve_params,
    config_instance_retrieve_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.config_instance import ConfigInstance
from ..types.config_instance_list_response import ConfigInstanceListResponse
from ..types.config_instance_deploy_response import ConfigInstanceDeployResponse

__all__ = ["ConfigInstancesResource", "AsyncConfigInstancesResource"]


class ConfigInstancesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConfigInstancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/miruml/python-server-sdk#accessing-raw-response-data-eg-headers
        """
        return ConfigInstancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigInstancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/miruml/python-server-sdk#with_streaming_response
        """
        return ConfigInstancesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        config_instance_id: str,
        *,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Get

        Args:
          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return self._get(
            f"/config_instances/{config_instance_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, config_instance_retrieve_params.ConfigInstanceRetrieveParams),
            ),
            cast_to=ConfigInstance,
        )

    def list(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        activity_status: Literal["created", "validating", "validated", "queued", "deployed", "removed"]
        | NotGiven = NOT_GIVEN,
        config_schema_id: str | NotGiven = NOT_GIVEN,
        config_type_id: str | NotGiven = NOT_GIVEN,
        device_id: str | NotGiven = NOT_GIVEN,
        error_status: Literal["none", "failed", "retrying"] | NotGiven = NOT_GIVEN,
        expand: List[Literal["total_count", "content", "config_schema", "device", "config_type"]]
        | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"] | NotGiven = NOT_GIVEN,
        target_status: Literal["created", "validated", "deployed", "removed"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstanceListResponse:
        """
        List config instances

        Args:
          id: The config instance ID to filter by

          activity_status: The config instance activity status to filter by

          config_schema_id: The config schema ID to filter by

          config_type_id: The config type ID to filter by

          device_id: The device ID to filter by

          error_status: The config instance error status to filter by

          expand: The fields to expand in the config instance list

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          target_status: The config instance target status to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/config_instances",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "activity_status": activity_status,
                        "config_schema_id": config_schema_id,
                        "config_type_id": config_type_id,
                        "device_id": device_id,
                        "error_status": error_status,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "target_status": target_status,
                    },
                    config_instance_list_params.ConfigInstanceListParams,
                ),
            ),
            cast_to=ConfigInstanceListResponse,
        )

    def approve(
        self,
        config_instance_id: str,
        *,
        message: str,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Approve a config instance for deployment

        Args:
          message: A high level success message displayed to the user

          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return self._post(
            f"/config_instances/{config_instance_id}/approve",
            body=maybe_transform({"message": message}, config_instance_approve_params.ConfigInstanceApproveParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, config_instance_approve_params.ConfigInstanceApproveParams),
            ),
            cast_to=ConfigInstance,
        )

    def deploy(
        self,
        config_instance_id: str,
        *,
        dry_run: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstanceDeployResponse:
        """
        Deploy a config instance to a device

        Args:
          dry_run: Perform a dry run that simulates the operation without making actual changes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return self._post(
            f"/config_instances/{config_instance_id}/deploy",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dry_run": dry_run}, config_instance_deploy_params.ConfigInstanceDeployParams),
            ),
            cast_to=ConfigInstanceDeployResponse,
        )

    def reject(
        self,
        config_instance_id: str,
        *,
        errors: Iterable[config_instance_reject_params.Error],
        message: str,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Reject a config instance for deployment

        Args:
          message: A high level error message displayed to the user

          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return self._post(
            f"/config_instances/{config_instance_id}/reject",
            body=maybe_transform(
                {
                    "errors": errors,
                    "message": message,
                },
                config_instance_reject_params.ConfigInstanceRejectParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"expand": expand}, config_instance_reject_params.ConfigInstanceRejectParams),
            ),
            cast_to=ConfigInstance,
        )


class AsyncConfigInstancesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConfigInstancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/miruml/python-server-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigInstancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigInstancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/miruml/python-server-sdk#with_streaming_response
        """
        return AsyncConfigInstancesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        config_instance_id: str,
        *,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Get

        Args:
          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return await self._get(
            f"/config_instances/{config_instance_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, config_instance_retrieve_params.ConfigInstanceRetrieveParams
                ),
            ),
            cast_to=ConfigInstance,
        )

    async def list(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        activity_status: Literal["created", "validating", "validated", "queued", "deployed", "removed"]
        | NotGiven = NOT_GIVEN,
        config_schema_id: str | NotGiven = NOT_GIVEN,
        config_type_id: str | NotGiven = NOT_GIVEN,
        device_id: str | NotGiven = NOT_GIVEN,
        error_status: Literal["none", "failed", "retrying"] | NotGiven = NOT_GIVEN,
        expand: List[Literal["total_count", "content", "config_schema", "device", "config_type"]]
        | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"] | NotGiven = NOT_GIVEN,
        target_status: Literal["created", "validated", "deployed", "removed"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstanceListResponse:
        """
        List config instances

        Args:
          id: The config instance ID to filter by

          activity_status: The config instance activity status to filter by

          config_schema_id: The config schema ID to filter by

          config_type_id: The config type ID to filter by

          device_id: The device ID to filter by

          error_status: The config instance error status to filter by

          expand: The fields to expand in the config instance list

          limit: The maximum number of items to return. A limit of 15 with an offset of 0 returns
              items 1-15.

          offset: The offset of the items to return. An offset of 10 with a limit of 10 returns
              items 11-20.

          target_status: The config instance target status to filter by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/config_instances",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "activity_status": activity_status,
                        "config_schema_id": config_schema_id,
                        "config_type_id": config_type_id,
                        "device_id": device_id,
                        "error_status": error_status,
                        "expand": expand,
                        "limit": limit,
                        "offset": offset,
                        "order_by": order_by,
                        "target_status": target_status,
                    },
                    config_instance_list_params.ConfigInstanceListParams,
                ),
            ),
            cast_to=ConfigInstanceListResponse,
        )

    async def approve(
        self,
        config_instance_id: str,
        *,
        message: str,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Approve a config instance for deployment

        Args:
          message: A high level success message displayed to the user

          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return await self._post(
            f"/config_instances/{config_instance_id}/approve",
            body=await async_maybe_transform(
                {"message": message}, config_instance_approve_params.ConfigInstanceApproveParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, config_instance_approve_params.ConfigInstanceApproveParams
                ),
            ),
            cast_to=ConfigInstance,
        )

    async def deploy(
        self,
        config_instance_id: str,
        *,
        dry_run: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstanceDeployResponse:
        """
        Deploy a config instance to a device

        Args:
          dry_run: Perform a dry run that simulates the operation without making actual changes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return await self._post(
            f"/config_instances/{config_instance_id}/deploy",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"dry_run": dry_run}, config_instance_deploy_params.ConfigInstanceDeployParams
                ),
            ),
            cast_to=ConfigInstanceDeployResponse,
        )

    async def reject(
        self,
        config_instance_id: str,
        *,
        errors: Iterable[config_instance_reject_params.Error],
        message: str,
        expand: List[Literal["content", "config_schema", "device", "config_type"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigInstance:
        """
        Reject a config instance for deployment

        Args:
          message: A high level error message displayed to the user

          expand: The fields to expand in the config instance

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_instance_id:
            raise ValueError(f"Expected a non-empty value for `config_instance_id` but received {config_instance_id!r}")
        return await self._post(
            f"/config_instances/{config_instance_id}/reject",
            body=await async_maybe_transform(
                {
                    "errors": errors,
                    "message": message,
                },
                config_instance_reject_params.ConfigInstanceRejectParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"expand": expand}, config_instance_reject_params.ConfigInstanceRejectParams
                ),
            ),
            cast_to=ConfigInstance,
        )


class ConfigInstancesResourceWithRawResponse:
    def __init__(self, config_instances: ConfigInstancesResource) -> None:
        self._config_instances = config_instances

        self.retrieve = to_raw_response_wrapper(
            config_instances.retrieve,
        )
        self.list = to_raw_response_wrapper(
            config_instances.list,
        )
        self.approve = to_raw_response_wrapper(
            config_instances.approve,
        )
        self.deploy = to_raw_response_wrapper(
            config_instances.deploy,
        )
        self.reject = to_raw_response_wrapper(
            config_instances.reject,
        )


class AsyncConfigInstancesResourceWithRawResponse:
    def __init__(self, config_instances: AsyncConfigInstancesResource) -> None:
        self._config_instances = config_instances

        self.retrieve = async_to_raw_response_wrapper(
            config_instances.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            config_instances.list,
        )
        self.approve = async_to_raw_response_wrapper(
            config_instances.approve,
        )
        self.deploy = async_to_raw_response_wrapper(
            config_instances.deploy,
        )
        self.reject = async_to_raw_response_wrapper(
            config_instances.reject,
        )


class ConfigInstancesResourceWithStreamingResponse:
    def __init__(self, config_instances: ConfigInstancesResource) -> None:
        self._config_instances = config_instances

        self.retrieve = to_streamed_response_wrapper(
            config_instances.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            config_instances.list,
        )
        self.approve = to_streamed_response_wrapper(
            config_instances.approve,
        )
        self.deploy = to_streamed_response_wrapper(
            config_instances.deploy,
        )
        self.reject = to_streamed_response_wrapper(
            config_instances.reject,
        )


class AsyncConfigInstancesResourceWithStreamingResponse:
    def __init__(self, config_instances: AsyncConfigInstancesResource) -> None:
        self._config_instances = config_instances

        self.retrieve = async_to_streamed_response_wrapper(
            config_instances.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            config_instances.list,
        )
        self.approve = async_to_streamed_response_wrapper(
            config_instances.approve,
        )
        self.deploy = async_to_streamed_response_wrapper(
            config_instances.deploy,
        )
        self.reject = async_to_streamed_response_wrapper(
            config_instances.reject,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BaseDevice"]


class BaseDevice(BaseModel):
    id: str
    """ID of the device"""

    created_at: datetime
    """Timestamp of when the device was created"""

    name: str
    """Name of the device"""

    object: Literal["device"]

    status: Literal["inactive", "activated", "provisioned"]
    """The status of the device

    - Inactive: The miru agent has not yet been installed / authenticated
    - Activated: The miru agent has been installed and authenticated
    - Provisioned: The device has been optionally initialized with config instances
    """

    updated_at: datetime
    """Timestamp of when the device was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)
from .config_instances import (
    ConfigInstancesResource,
    AsyncConfigInstancesResource,
    ConfigInstancesResourceWithRawResponse,
    AsyncConfigInstancesResourceWithRawResponse,
    ConfigInstancesResourceWithStreamingResponse,
    AsyncConfigInstancesResourceWithStreamingResponse,
)

__all__ = [
    "ConfigInstancesResource",
    "AsyncConfigInstancesResource",
    "ConfigInstancesResourceWithRawResponse",
    "AsyncConfigInstancesResourceWithRawResponse",
    "ConfigInstancesResourceWithStreamingResponse",
    "AsyncConfigInstancesResourceWithStreamingResponse",
    "DevicesResource",
    "AsyncDevicesResource",
    "DevicesResourceWithRawResponse",
    "AsyncDevicesResourceWithRawResponse",
    "DevicesResourceWithStreamingResponse",
    "AsyncDevicesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConfigInstanceRejectParams", "Error"]


class ConfigInstanceRejectParams(TypedDict, total=False):
    errors: Required[Iterable[Error]]

    message: Required[str]
    """A high level error message displayed to the user"""

    expand: List[Literal["content", "config_schema", "device", "config_type"]]
    """The fields to expand in the config instance"""


class Error(TypedDict, total=False):
    message: Required[str]
    """A detailed message explaining why the parameter is causing the error"""

    parameter_path: Required[List[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from miru_server_sdk import Miru, AsyncMiru
from miru_server_sdk.types import (
    ConfigInstance,
    ConfigInstanceListResponse,
    ConfigInstanceDeployResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConfigInstances:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Miru) -> None:
        config_instance = client.config_instances.retrieve(
            config_instance_id="cfg_inst_123",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Miru) -> None:
        config_instance = client.config_instances.retrieve(
            config_instance_id="cfg_inst_123",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Miru) -> None:
        response = client.config_instances.with_raw_response.retrieve(
            config_instance_id="cfg_inst_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Miru) -> None:
        with client.config_instances.with_streaming_response.retrieve(
            config_instance_id="cfg_inst_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            client.config_instances.with_raw_response.retrieve(
                config_instance_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Miru) -> None:
        config_instance = client.config_instances.list()
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Miru) -> None:
        config_instance = client.config_instances.list(
            id="cfg_inst_123",
            activity_status="created",
            config_schema_id="cfg_sch_123",
            config_type_id="cfg_typ_123",
            device_id="dvc_123",
            error_status="none",
            expand=["total_count"],
            limit=1,
            offset=0,
            order_by="id:asc",
            target_status="created",
        )
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Miru) -> None:
        response = client.config_instances.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = response.parse()
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Miru) -> None:
        with client.config_instances.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = response.parse()
            assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_approve(self, client: Miru) -> None:
        config_instance = client.config_instances.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_approve_with_all_params(self, client: Miru) -> None:
        config_instance = client.config_instances.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_approve(self, client: Miru) -> None:
        response = client.config_instances.with_raw_response.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_approve(self, client: Miru) -> None:
        with client.config_instances.with_streaming_response.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_approve(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            client.config_instances.with_raw_response.approve(
                config_instance_id="",
                message="The config instance has been approved",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_deploy(self, client: Miru) -> None:
        config_instance = client.config_instances.deploy(
            config_instance_id="cfg_inst_123",
        )
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_deploy_with_all_params(self, client: Miru) -> None:
        config_instance = client.config_instances.deploy(
            config_instance_id="cfg_inst_123",
            dry_run=True,
        )
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_deploy(self, client: Miru) -> None:
        response = client.config_instances.with_raw_response.deploy(
            config_instance_id="cfg_inst_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = response.parse()
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_deploy(self, client: Miru) -> None:
        with client.config_instances.with_streaming_response.deploy(
            config_instance_id="cfg_inst_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = response.parse()
            assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_deploy(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            client.config_instances.with_raw_response.deploy(
                config_instance_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_reject(self, client: Miru) -> None:
        config_instance = client.config_instances.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_reject_with_all_params(self, client: Miru) -> None:
        config_instance = client.config_instances.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_reject(self, client: Miru) -> None:
        response = client.config_instances.with_raw_response.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_reject(self, client: Miru) -> None:
        with client.config_instances.with_streaming_response.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_reject(self, client: Miru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            client.config_instances.with_raw_response.reject(
                config_instance_id="",
                errors=[
                    {
                        "message": "Motion detection sensitivity must be between 0 and 100",
                        "parameter_path": ["motion_detection", "sensitivity"],
                    }
                ],
                message="The config instance contains additional parameters that are not supported by the config schema",
            )


class TestAsyncConfigInstances:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.retrieve(
            config_instance_id="cfg_inst_123",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.retrieve(
            config_instance_id="cfg_inst_123",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncMiru) -> None:
        response = await async_client.config_instances.with_raw_response.retrieve(
            config_instance_id="cfg_inst_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = await response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncMiru) -> None:
        async with async_client.config_instances.with_streaming_response.retrieve(
            config_instance_id="cfg_inst_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = await response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            await async_client.config_instances.with_raw_response.retrieve(
                config_instance_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.list()
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.list(
            id="cfg_inst_123",
            activity_status="created",
            config_schema_id="cfg_sch_123",
            config_type_id="cfg_typ_123",
            device_id="dvc_123",
            error_status="none",
            expand=["total_count"],
            limit=1,
            offset=0,
            order_by="id:asc",
            target_status="created",
        )
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncMiru) -> None:
        response = await async_client.config_instances.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = await response.parse()
        assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncMiru) -> None:
        async with async_client.config_instances.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = await response.parse()
            assert_matches_type(ConfigInstanceListResponse, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_approve(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_approve_with_all_params(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_approve(self, async_client: AsyncMiru) -> None:
        response = await async_client.config_instances.with_raw_response.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = await response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_approve(self, async_client: AsyncMiru) -> None:
        async with async_client.config_instances.with_streaming_response.approve(
            config_instance_id="cfg_inst_123",
            message="The config instance has been approved",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = await response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_approve(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            await async_client.config_instances.with_raw_response.approve(
                config_instance_id="",
                message="The config instance has been approved",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_deploy(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.deploy(
            config_instance_id="cfg_inst_123",
        )
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_deploy_with_all_params(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.deploy(
            config_instance_id="cfg_inst_123",
            dry_run=True,
        )
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_deploy(self, async_client: AsyncMiru) -> None:
        response = await async_client.config_instances.with_raw_response.deploy(
            config_instance_id="cfg_inst_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = await response.parse()
        assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_deploy(self, async_client: AsyncMiru) -> None:
        async with async_client.config_instances.with_streaming_response.deploy(
            config_instance_id="cfg_inst_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = await response.parse()
            assert_matches_type(ConfigInstanceDeployResponse, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_deploy(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            await async_client.config_instances.with_raw_response.deploy(
                config_instance_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_reject(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_reject_with_all_params(self, async_client: AsyncMiru) -> None:
        config_instance = await async_client.config_instances.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
            expand=["content"],
        )
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_reject(self, async_client: AsyncMiru) -> None:
        response = await async_client.config_instances.with_raw_response.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config_instance = await response.parse()
        assert_matches_type(ConfigInstance, config_instance, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_reject(self, async_client: AsyncMiru) -> None:
        async with async_client.config_instances.with_streaming_response.reject(
            config_instance_id="cfg_inst_123",
            errors=[
                {
                    "message": "Motion detection sensitivity must be between 0 and 100",
                    "parameter_path": ["motion_detection", "sensitivity"],
                }
            ],
            message="The config instance contains additional parameters that are not supported by the config schema",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config_instance = await response.parse()
            assert_matches_type(ConfigInstance, config_instance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_reject(self, async_client: AsyncMiru) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_instance_id` but received ''"):
            await async_client.config_instances.with_raw_response.reject(
                config_instance_id="",
                errors=[
                    {
                        "message": "Motion detection sensitivity must be between 0 and 100",
                        "parameter_path": ["motion_detection", "sensitivity"],
                    }
                ],
                message="The config instance contains additional parameters that are not supported by the config schema",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

from .paginated_list import PaginatedList

__all__ = ["ConfigInstanceListResponse"]


class ConfigInstanceListResponse(PaginatedList):
    data: List["ConfigInstance"]


from .config_instance import ConfigInstance



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .base_device import BaseDevice
from .paginated_list import PaginatedList

__all__ = ["DeviceListResponse"]


class DeviceListResponse(PaginatedList):
    data: Optional[List[BaseDevice]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConfigInstanceDeployResponse", "Remove"]


class Remove(BaseModel):
    code: Literal["device_file_path_conflict", "device_config_schema_conflict"]
    """The reason the config instance was removed

    - device_file_path_conflict: the config instance is currently deployed to the
      same device with the same file path as the new config instance to deploy
    - device_config_schema_conflict: the config instance is currently deployed to
      the same device with the same config schema as the new config instance to
      deploy
    """

    config_instance: "ConfigInstance"

    explanation: str
    """
    A human-readable explanation of why the config instance must be removed for the
    new config instance to be able to be deployed
    """


class ConfigInstanceDeployResponse(BaseModel):
    deploy: "ConfigInstance"

    remove: List[Remove]


from .config_instance import ConfigInstance



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ConfigInstanceDeployParams"]


class ConfigInstanceDeployParams(TypedDict, total=False):
    dry_run: bool
    """Perform a dry run that simulates the operation without making actual changes"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConfigSchema"]


class ConfigSchema(BaseModel):
    id: str
    """ID of the config schema"""

    config_type: Optional["ConfigType"] = None
    """Expand the config type using 'expand[]=config_type' in the query string"""

    config_type_id: str
    """ID of the config type"""

    content: Optional[object] = None
    """The config schema's JSON Schema definition"""

    created_at: datetime
    """Timestamp of when the config schema was created"""

    digest: str
    """Hash of the config schema contents"""

    object: Literal["config_schema"]

    relative_filepath: str
    """
    The file path to deploy the config instance relative to
    `/srv/miru/config_instances`. `v1/motion-control.json` would deploy to
    `/srv/miru/config_instances/v1/motion-control.json`
    """

    updated_at: datetime
    """Timestamp of when the config schema was last updated"""

    version: int
    """Config schema version for the config type"""


from .config_type import ConfigType



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["DeviceListParams"]


class DeviceListParams(TypedDict, total=False):
    id: str
    """The device ID to filter by"""

    expand: List[Literal["total_count"]]
    """The fields to expand in the device list"""

    limit: int
    """The maximum number of items to return.

    A limit of 15 with an offset of 0 returns items 1-15.
    """

    name: str
    """The device name to filter by"""

    offset: int
    """The offset of the items to return.

    An offset of 10 with a limit of 10 returns items 11-20.
    """

    order_by: Literal["id:asc", "id:desc", "created_at:desc", "created_at:asc"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConfigType"]


class ConfigType(BaseModel):
    id: str
    """ID of the config type"""

    config_schemas: Optional["ConfigSchemaList"] = None
    """Expand the config schemas using 'expand[]=schemas' in the query string"""

    created_at: datetime
    """Timestamp of when the config type was created"""

    name: str
    """Name of the config type"""

    object: Literal["config_type"]

    slug: str
    """An immutable, code-friendly name for the config type"""

    updated_at: datetime
    """Timestamp of when the config type was last updated"""


from .config_schema_list import ConfigSchemaList



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

from .paginated_list import PaginatedList

__all__ = ["ConfigSchemaList"]


class ConfigSchemaList(PaginatedList):
    data: List["ConfigSchema"]


from .config_schema import ConfigSchema
